#include <libwccl/ops/functions/bool/iteration.h>
#include <libwccl/ops/functions/bool/predicate.h>
#include <sstream>

namespace Wccl {

std::string Iteration::to_string(const Corpus2::Tagset& tagset) const
{
	std::ostringstream ss;
	ss << name(tagset) << "("
		<< left_pos_expr_->to_string(tagset) << ", "
		<< right_pos_expr_->to_string(tagset) << ", "
		<< Position::var_repr(iter_var_acc_.get_name()) << ", "
		<< evaluating_expr_->to_string(tagset) << ")";
	return ss.str();
}

std::string Iteration::to_raw_string() const
{
	std::ostringstream ss;
	ss << raw_name() << "("
		<< left_pos_expr_->to_raw_string() << ", "
		<< right_pos_expr_->to_raw_string() << ", "
		<< Position::var_repr(iter_var_acc_.get_name()) << ", "
		<< evaluating_expr_->to_raw_string() << ")";
	return ss.str();
}

Iteration::BaseRetValPtr Iteration::apply_internal(const FunExecContext& context) const
{
	const boost::shared_ptr<const Position>& range_begin = left_pos_expr_->apply(context);
	const boost::shared_ptr<const Position>& range_end = right_pos_expr_->apply(context);
	const boost::shared_ptr<Position>& iter_var = context.variables()->get_fast(iter_var_acc_);

	const SentenceContext& sc = context.sentence_context();
	// Get absolute values for left and right extremes of the range
	int left = sc.get_abs_position(*range_begin);
	int right = sc.get_abs_position(*range_end);
	// Proceed only if range extremes are not "nowhere".
	if (left != Position::Nowhere && right != Position::Nowhere) {
		// Trim range to sentence boundaries
		if (left < 0) {
			left = 0;
		}
		if (right >= sc.size()) {
			right = sc.size() - 1;
		}
		// Proceed only if range isn't empty
		if (left <= right) {
			// Change range from absolute to relative and iterate
			left -= sc.get_position();
			right -= sc.get_position();
			if (iterate(left, right, *iter_var, context)) {
				return Predicate::True(context);
			}
		}
	}
	// In case of failure, set iteration variable to Nowhere and return False
	iter_var->set_value(Position::Nowhere);
	return Predicate::False(context);
}

} /* end ns Wccl */
